Attribute VB_Name = "modMain"
Option Explicit

Private m_LocalHostName As String
Private m_RemoteHostName As String
Private m_FileObject As FILECTLCtl.File

' the Winsock component doesn't expose these values
Public Const sckIRDAProtocol = 2
Public Const sckTCPProtocol = 0

Public Function GetFileObject() As FILECTLCtl.File
    On Error Resume Next
    
    ' If we haven't created the File object yet, do so
    If IsEmpty(m_FileObject) Then
        ' Create a File control
        Set m_FileObject = CreateObject("FILECTL.File")
        
        ' Ensure the object was created successfully
        If Err.number <> 0 Then
            MsgBox "Failed to create File object." & vbCrLf & _
                "Ensure MSCEFile.dll has been installed and registered.", _
                vbCritical, "Error"
            Exit Function
        End If
    End If
    
    ' Return our global File object
    Set GetFileObject = m_FileObject
End Function

Public Sub ReleaseFileObject()
    Set m_FileObject = Nothing
End Sub

Public Function GetFileTextContents(Path As String, ByRef Contents As String) As Boolean
    Dim filFile As FILECTLCtl.File
    
    On Error Resume Next
    
    ' Set our return value
    GetFileTextContents = True
    
    ' Get our application File object
    Set filFile = GetFileObject()
    
    ' Open the File
    filFile.Open Path, fsModeInput, fsAccessRead
    
    ' Make sure the call to Open was successful
    If Err.number <> 0 Then
        GetFileTextContents = False
        Exit Function
    End If
    
    ' Loop through file, filling our input buffer
    Do While Not filFile.EOF
        Contents = Contents & filFile.Input(1)
    Loop
    
    ' Close the file
    filFile.Close
    
    ' Release the File Object
    Set filFile = Nothing
End Function

Public Sub WriteFile(FileString As String)
    Dim filFile As FILECTL.File
    Dim strFileName As String
    Dim iEndPos As Integer
    
    Set filFile = GetFileObject()
    
    ' First strip the tags
    FileString = Replace(FileString, "</PocketChat File>", "")
    FileString = Replace(FileString, "<PocketChat File>", "")
    
    ' Next get the filename
    FileString = Replace(FileString, "<Filename>", "")
    iEndPos = InStr(FileString, "</Filename>")
    strFileName = Left(FileString, iEndPos - 1)
    
    ' Strip the filename and tags from the file contents
    FileString = Mid(FileString, iEndPos + 11)
    
    ' Open the file
    filFile.Open App.Path & "\" & strFileName, fsModeOutput
    
    ' Write the data
    filFile.LinePrint FileString
    
    ' Close the file
    filFile.Close
End Sub

' the LocalHostName "Property"
Public Function GetLocalHostName() As String
    GetLocalHostName = m_LocalHostName
End Function

Public Sub SetLocalHostName(NewName As String)
    m_LocalHostName = NewName
End Sub

' the RemoteHostName "Property"
Public Function GetRemoteHostName() As String
    GetRemoteHostName = m_RemoteHostName
End Function

Public Sub SetRemoteHostName(NewName As String)
    m_RemoteHostName = NewName
End Sub

Public Sub SendFile()
    Dim strPath As String
    Dim strFileName As String
    Dim strContents As String
    Dim iStartPos As Integer
    
    strPath = InputBox("Enter name of file to send (include full path)", "Send File")
    
    ' if the user hit Cancel of OK with a blank input, exit
    If strPath = "" Then Exit Sub
    
    ' Refresh frmMain to get rid of InputBox "ghost"
    frmMain.Refresh
    
    ' Get the file's contents
    If GetFileTextContents(strPath, strContents) = True Then
        ' Bracket file contents in tags
        ' Add filename tags
        iStartPos = InStrRev(strPath, "\")
        If iStartPos <= 0 Then iStartPos = 1
        strFileName = Mid(strPath, iStartPos)
        
        ' Inform the user we're beginning and hide the SIP
        frmMain.txtRecv = frmMain.txtRecv & "Sending file: " & strFileName & vbCrLf
        frmMain.Refresh
        
        ' Start building data stream
        strContents = "<Filename>" & strFileName & "</Filename>" & strContents
        
        ' Add PocketChat tags
        strContents = "<PocketChat File>" & strContents & "</PocketChat File>"
        
        ' Send the file
        frmMain.wskMain.SendData strContents
        
        ' Inform the user we've sent the file
        frmMain.txtRecv.Text = frmMain.txtRecv.Text & "Sent File: " _
            & strFileName & vbCrLf
    Else
        ' failed to get contents.  Likely file doesn't exist
        MsgBox "Cannot locate file to send", vbCritical, "Error sending"
    End If
    
End Sub

Public Sub SendText(strSend As String)
    Dim ilength As Integer
    
    ' Append our local name
    strSend = GetLocalHostName & ": " & strSend
    
    ' Send the data with our custom tags
    frmMain.wskMain.SendData "<PocketChat Text>" & strSend & "</PocketChat Text>"
    
    ' Move it to the "recv" textbox
    frmMain.txtRecv.Text = frmMain.txtRecv.Text & strSend & vbCrLf
    
    ' Clear the "Send" textbox
    frmMain.txtSend.Text = ""
    
    ' Set focus back to the send textbox
    frmMain.txtSend.SetFocus
End Sub

Public Sub Main()
    ' Set our protocol to TCP/IP
    frmMain.wskMain.Protocol = sckTCPProtocol
    
    ' Get our local host name and set the property
    SetLocalHostName frmMain.wskMain.LocalHostName
        
    ' Show frmMain
    frmMain.Show
    
    ' We'll assume we're a socket server to begin with
    ' So we put the socket into listen mode
    frmMain.wskMain.Listen
    
    ' Set up the MenuBar
    frmMain.MenuBarInit
End Sub

Public Sub DrawProgress(picProgress As PictureBoxCtl.PictureBox, PercentDone As Integer, DrawColor As Long)
    Dim sngPercentWidth As Single
    Dim sngDrawWidth As Single
    
    ' Check for valid input
    If PercentDone < 0 Then PercentDone = 0
    If PercentDone > 100 Then PercentDone = 100
    
    ' Allow for decimal representation of percent
    If PercentDone < 1 Then PercentDone = PercentDone * 100
    
    ' Determine the width of one percent
    sngPercentWidth = picProgress.Width / 100
    
    ' Determine the width to draw
    sngDrawWidth = PercentDone * sngPercentWidth
    
    ' Fill the picturebox in White
    If PercentDone = 0 Then
        picProgress.DrawLine 0, 0, picProgress.Width, _
            picProgress.Height, RGB(255, 255, 255), True, True
    End If
        
    ' Draw the progress bar - essentially a filled rectangle
    picProgress.DrawLine 0, 0, sngDrawWidth, _
        picProgress.Height, DrawColor, True, True
    
    ' Without Refresh, the progress is not displayed until done
    picProgress.Parent.Refresh
End Sub
