Attribute VB_Name = "modGlobal"
Option Explicit

' constants for mode that the
' cd form can operate in
Public Const CDFORM_MODE_EDIT = 0
Public Const CDFORM_MODE_NEW = 1

' contstants for the ADOCE classes
Public Const ADOCE_CLASS_CONNECTION = "ADOCE.Connection.3.1"
Public Const ADOCE_CLASS_RECORDSET = "ADOCE.Recordset.3.1"

' contstant for database path and name
Const CD_CDB_PATH_FILENAME = "\cd.cdb"

' global objects for all ADOCE objects
Public gobjConnection As ADOCE.Connection
Public gobjArtist As ADOCE.Recordset
Public gobjCD As ADOCE.Recordset
Public gobjPKEY As ADOCE.Recordset
Public gobjGenre As ADOCE.Recordset

Function InitializeGlobals() As Boolean

    On Error Resume Next
    
    InitializeGlobals = False
    
    ' initialize globals - this needs to take place
    ' only once, due to CreateObject memory leak
    
    ' ADOCE_CLASS_CONNECTION & ADOCE_CLASS_RECORDSET
    ' are used because ce references these classes
    ' by their version numbers, create Consts allows
    ' changing of these version quickly and accurately
    Set gobjConnection = CreateObject(ADOCE_CLASS_CONNECTION)
    
    Set gobjArtist = CreateObject(ADOCE_CLASS_RECORDSET)
    Set gobjCD = CreateObject(ADOCE_CLASS_RECORDSET)
    Set gobjGenre = CreateObject(ADOCE_CLASS_RECORDSET)
    Set gobjPKEY = CreateObject(ADOCE_CLASS_RECORDSET)
    
    Set gConfigRecord = CreateObject(ADOCE_CLASS_RECORDSET)
    
    Set gSQLReplication = CreateObject(SSCE_REPLICATION_CLASS)
    
    Set gobjADOXCatalog = CreateObject(ADOXCE_CATALOG_CLASS)
    
    Set gobjUser = CreateObject(ADOCE_CLASS_RECORDSET)
    
    If Err.Number = 0 Then
        InitializeGlobals = True
    End If

End Function

Function ReleaseGlobals() As Boolean

    On Error Resume Next
    
    ReleaseGlobals = True
    
    If gobjConnection.State = adStateOpen Then
        gobjConnection.Close
    End If
    
    'release all global object
    Set gobjConnection = Nothing
    Set gobjArtist = Nothing
    Set gobjCD = Nothing
    Set gobjPKEY = Nothing
    Set gobjGenre = Nothing
    
    Set gConfigRecord = Nothing
    Set gobjADOXCatalog = Nothing
    Set gobjUser = Nothing
    
    If Err.Number = 0 Then
        ReleaseGlobals = True
    End If

End Function
Function CloseDatabase() As Boolean
    
    On Error Resume Next
    
    CloseDatabase = False
    
    If Not gobjConnection Is Nothing Then
        If gobjConnection.State = adStateOpen Then
            gobjConnection.Close
        End If
    End If
        
    If Err.Number = 0 Then
        CloseDatabase = True
    End If

End Function
Function OpenDatabase() As Boolean

    On Error Resume Next
    
    ' set the return value to an error
    ' condition, so we can exit early
    ' if an error occurs
    OpenDatabase = False
    
    ' we need to check if the connection is
    ' open or close, because EnsureDatabaseExists may
    ' opened the database
    
    If gobjConnection.State = adStateClosed Then
        ' opens the SQL Server CE database
        gobjConnection.Open "Provider=Microsoft.SQLServer.OLEDB.CE.1.0;" _
            & "Data Source=" & DB_PATH
            
        ' if we're opening the db
        ' let's set the UserId
        If Not SetUserId() Then
            Exit Function
        End If
    End If
    
    If Err.Number = 0 Then
        OpenDatabase = True
    End If

End Function

Function AddGenre(ByVal strGenre As String) As Long

    ' adds a genre to the table and returns
    ' its primary key value
    
    On Error Resume Next
    
    ' set the return value to error code
    ' in case we need to exit early
    AddGenre = -1
    
    ' if the recordset is open
    ' let's check if it supports addnew
    If gobjGenre.State = adStateOpen Then
        ' if the current recordset does not
        ' support adding new records, let's close
        ' it and reopen it
        If Not gobjGenre.Supports(adAddNew) Then
            gobjGenre.Close
        End If
    End If
    
    ' if the recordset is closed above
    ' or closed before we get into this
    ' function, let's open it with the
    ' proper support for adding
    If gobjGenre.State = adStateClosed Then
        gobjGenre.Open "genre", gobjConnection, _
            adOpenDynamic, adLockOptimistic, adCmdTableDirect
    End If
    
    ' add a new record
    gobjGenre.AddNew
    
    ' set the values
    gobjGenre.Fields("GenreName").Value = strGenre
    
    ' update the recordset
    gobjGenre.Update

    ' set the return value
    gobjGenre.Close
    
    gobjGenre.Open "SELECT MAX( GenreId ) AS GenreId FROM Genre", _
        gobjConnection, adOpenStatic, adLockOptimistic
    
    AddGenre = CLng(gobjGenre.Fields("GenreId").Value)
    
End Function

Function AddArtist(ByVal strArtist As String, ByVal lngDefaultId As Long) As Long

    ' adds an artist to the table and returns
    ' its primary key value
    ' if there is an error, returns a negative value
    
    Dim lngArtistId As Long
    
    On Error Resume Next
    
    ' set the return value to an error value
    ' in case we exit early
    AddArtist = -1
    
    ' needs to test the recordset to see if
    ' it currently supports adding new records
    
    ' let's see if recordset is open
    If gobjArtist.State = adStateOpen Then
        ' if it currently does not support
        ' adding new, let's close it and reopen
        ' it below
        If Not gobjArtist.Supports(adAddNew) Then
            ' close the recordset
            gobjArtist.Close
        End If
    End If
    
    ' if it was closed before we got into this
    ' function or closed in this function, let's
    ' reopen it with the correct support (AddsNew)
    If gobjArtist.State = adStateClosed Then
        gobjArtist.Open _
            "artist", _
            gobjConnection, _
            adOpenDynamic, _
            adLockOptimistic, _
            adCmdTableDirect
        ' "artist" - open the table direct
        ' gobjConnection - the global connection object
        ' adOpenForwardOnly - we're only going to add,
        '   so no need to have scrollable cursor
        ' adLockOptimistic - quick updates
        ' adCmdTableDirect - optional parameter that
        ' informs the recordset to open the table directly
        ' allowing the quickest manipulation
    End If
    
    
    ' append a new record
    gobjArtist.AddNew
    
    ' set values
    gobjArtist.Fields("ArtistName").Value = strArtist
    
    ' if we have a genre, let's set it
    If Not IsEmpty(lngDefaultId) Then
        gobjArtist.Fields("DefaultGenre").Value = lngDefaultId
    End If
    
    ' update the recordset - commits the record
    gobjArtist.Update
    
    gobjArtist.Close
    
    gobjArtist.Open "SELECT MAX( ArtistId ) AS ArtistId FROM Artist", gobjConnection, adOpenStatic, adLockReadOnly
    
    ' if we had an error, let's set the
    ' return value to a negative value
    ' so we know that it failed
    If Err.Number = 0 Then
        ' set the return value
        AddArtist = CLng(gobjArtist.Fields("ArtistId").Value)
    End If

End Function
